==> tkit-quarkus-rhpam-client-0.0.14-sources.jar: org/tkit/rhpam/quarkus/messaging/FailedStepService.java <==
package org.tkit.rhpam.quarkus.messaging;

import ext.api.centrallog.api.ProcessLogEventEmitter;
import ext.api.centrallog.model.*;
import io.smallrye.reactive.messaging.amqp.AmqpMessage;
import io.smallrye.reactive.messaging.amqp.IncomingAmqpMetadata;
import io.vertx.core.json.JsonObject;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.eclipse.microprofile.reactive.messaging.Message;
import org.hibernate.service.spi.ServiceException;
import org.tkit.rhpam.quarkus.messaging.common.Constants;
import org.tkit.rhpam.quarkus.messaging.common.MessageUtil;
import org.tkit.rhpam.quarkus.messaging.common.RhpamException;
import org.tkit.rhpam.quarkus.messaging.model.ProcessStepExecution;
import org.tkit.rhpam.quarkus.messaging.model.ProcessStepExecutionResult;
import org.tkit.rhpam.quarkus.processlog.domain.daos.FailedStepDAO;
import org.tkit.rhpam.quarkus.processlog.domain.models.FailedStep;
import org.tkit.rhpam.quarkus.processlog.domain.models.FailedStepSearchCriteria;

import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import java.io.IOException;
import java.util.*;

/**
 * The type Failed step service.
 */
@ApplicationScoped
@Slf4j
public class FailedStepService {
    private static final String SUPPORT_TOOL_CALLBACK_URL_PROPERTY = "ORG_TKIT_RHPAM_SUPPORT_TOOL_CALLBACK_URL";
    @Inject
    ProcessLogEventEmitter centralLogEmitter;

    @ConfigProperty(name = "quarkus.resteasy.path")
    private String contextPath;

    @Inject
    FailedStepDAO service;

    public FailedStep createOrUpdateFailedStep(AmqpMessage<String> message) throws ServiceException {
        try {
            FailedStep failedStep = getExistingFailedStep(message);
            if (failedStep != null) {
                failedStep = service.update(failedStep);
            } else {
                failedStep = constructFailedStepFromMessage(message);
                failedStep = service.create(failedStep);
            }

            return failedStep;

        } catch (Exception e) {
            throw new IllegalStateException(FailedStepServiceErrorKeys.FAILED_TO_CREATE_FAILED_STEP.name(), e);
        }
    }

    public void emitFailedStepEvent(FailedStep failedStep) throws  RhpamException {
        OpenIncidentEvent event = new OpenIncidentEvent();
        event.setErrorCode(failedStep.getErrorCode());
        event.setErrorMessage(failedStep.getErrorMessage());
        event.setErrorType(ErrorType.fromValue(failedStep.getFailureType().name()));
        event.setNodeId(failedStep.getNodeId());
        event.setExternalIssueId(failedStep.getExternalIssueId());
        event.setPriority(PriorityLevel.HIGH);
        List<ResolutionOption> options = new ArrayList<>();
        String[] optionArr = failedStep.getResolutionOptionsString().split(",");
        for (String item : optionArr) {
            options.add(new ResolutionOption().key(item).label(item).callBackUrl(constructResolutionCallbackUrlForCentralLog(failedStep.getId())));
        }
        event.setResolutionOptions(options);
        event.setRetryCount(failedStep.getExecutionCount());
        event.setCorrelationId(failedStep.getId());
        event.setEventTime(failedStep.getCreationDate());
        event.setIncidentId(failedStep.getId());
        event.setProcessEventType(EventType.OPEN_INCIDENT_EVENT);
        event.setProcessId(failedStep.getProcessId());
        event.setProcessInstanceId(String.valueOf(failedStep.getProcessInstanceId()));
        event.setExecutionId(String.valueOf(failedStep.getWorkItemId()));
        event.getMetadata().put("deepLink", failedStep.getDeepLink());
        event.getMetadata().put("deploymentId", failedStep.getDeploymentId());
        event.getMetadata().put("referenceKey", failedStep.getReferenceKey());
        event.getMetadata().put("referenceBid", failedStep.getReferenceBid());
        event.getMetadata().put("processStepName", failedStep.getProcessStepName());
        centralLogEmitter.emitProcessEvent(event);
    }

    /**
     * Create Failed step for a process step that was technically executed ok, but resulted in business error
     *
     * @param workItem        process execution data
     * @param executionResult process execution result data
     * @return created failed step
     * @throws ServiceException
     */
    public FailedStep createOrUpdateBusinessFailedStep(ProcessStepExecution workItem, ProcessStepExecutionResult executionResult) throws ServiceException, IOException {
        FailedStep failedStep = null;
        FailedStepSearchCriteria criteria = new FailedStepSearchCriteria();
        criteria.setFailureType(FailedStep.FailureType.BUSINESS_ERROR);
        criteria.setProcessStepName(workItem.getName());
        criteria.setProcessInstanceId(workItem.getProcessInstanceId());
        List<FailedStep> failedStepList = service.findBySearchCriteria(criteria);
        // a failed step for this error already exists so update the status
        if (failedStepList != null && failedStepList.size() == 1) {
            failedStep = failedStepList.get(0);
            failedStep.setExecutionCount(failedStep.getExecutionCount() + 1);
            failedStep.setStatus(FailedStep.FailedStepStatus.OPEN);
            failedStep.setWorkItemId(workItem.getWorkItemId());
            return service.update(failedStep);
        } else {
            failedStep = new FailedStep();
            failedStep.setFailureType(FailedStep.FailureType.BUSINESS_ERROR);

            //failedStep.setResolutionOptionsString(resolutionOptions);
            //failedStep.setResolutionSignalReceiver(resolutionSignalId);
            failedStep.setExecutionCount(1);
            failedStep.setProcessInstanceId(workItem.getProcessInstanceId());

            if (executionResult.getParameters().get(Constants.FAILED_STEP_ERROR_CODE) != null) {
                failedStep.setErrorCode((String) executionResult.getParameters().get(Constants.FAILED_STEP_ERROR_CODE));
            }
            if (executionResult.getParameters().get(Constants.FAILED_STEP_ERROR_MSG) != null) {
                failedStep.setErrorMessage((String) executionResult.getParameters().get(Constants.FAILED_STEP_ERROR_MSG));
            }
            if (executionResult.getParameters().get(Constants.FAILED_STEP_DEEP_LINK) != null) {
                failedStep.setDeepLink((String) executionResult.getParameters().get(Constants.FAILED_STEP_DEEP_LINK));
            }
            failedStep.setNodeId(workItem.getNodeId());
            failedStep.setNodeName(workItem.getName());
            failedStep.setProcessId(workItem.getProcessId());
            failedStep.setProcessInstanceId(workItem.getProcessInstanceId());
            failedStep.setParentProcessInstanceId(workItem.getParentProcessInstanceId() != null ? String.valueOf(workItem.getParentProcessInstanceId()) : null);
            failedStep.setProcessStepName(workItem.getName());
            failedStep.setProcessName(workItem.getProcessName());
            failedStep.setStatus(FailedStep.FailedStepStatus.OPEN);
            failedStep.setWorkItemId(workItem.getWorkItemId());
            failedStep.setReferenceBid(workItem.getReferenceId());
            failedStep.setReferenceKey(workItem.getReferenceKey());
            failedStep.setDeploymentId(workItem.getDeploymentId());
            //TODO hack, we misuses the field, to store the output params, because we do not have other option
            failedStep.setOriginalJMSMessage(MessageUtil.serializeBody(executionResult.getParameters()));
            return service.create(failedStep);
        }
    }

    public FailedStep updateFailedStep(ProcessStepExecution workItem, String referencedStepName, String resolutionOptions, String resolutionSignalId) throws ServiceException {
        FailedStep failedStep = null;
        FailedStepSearchCriteria criteria = new FailedStepSearchCriteria();
        criteria.setFailureType(FailedStep.FailureType.BUSINESS_ERROR);
        criteria.setProcessStepName(referencedStepName);
        criteria.setProcessInstanceId(workItem.getProcessInstanceId());
        List<FailedStep> failedStepList = service.findBySearchCriteria(criteria);
        // a failed step for this error already exists so update the status
        if (failedStepList != null && failedStepList.size() == 1) {
            failedStep = failedStepList.get(0);
        } else {
            throw new IllegalArgumentException("Business error handler invoked but no matching failed step found! Step: " +  referencedStepName);
//            failedStep = new FailedStep();
//            failedStep.setProcessId(workItem.getProcessId());
//            failedStep.setProcessInstanceId(workItem.getProcessInstanceId());
//            failedStep.setErrorCode("TKIT_RHPAM_INVALID_BUSINESS_ERROR_HANDLER");
//            failedStep.setFailureType(FailureType.TECHNICAL_ERROR);
//            failedStep.setExecutionCount(1);
//            failedStep.setReferenceBid(workItem.getReferenceId());
//            failedStep.setReferenceKey(workItem.getReferenceKey());
//            failedStep.setNodeId(workItem.getNodeId());
//            failedStep.setNodeName(workItem.getName());
//            failedStep.setStatus(FailedStepStatus.OPEN);
//            failedStep.setWorkItemId(workItem.getWorkItemId());
        }
        failedStep.setResolutionOptionsString(resolutionOptions);
        failedStep.setResolutionSignalReceiver(resolutionSignalId);
        failedStep = service.update(failedStep);

        return failedStep;
    }

    /**
     * Try to find the FailedStep object
     *
     * @param msg failed step Message
     * @return found FailedStep based on workItemId or null with updated status
     */
    private FailedStep getExistingFailedStep(Message msg) {
        FailedStep failedStep = null;

        try {

            Long workItemId = MessageUtil.getProperty(msg, MessageUtil.PROP_WORK_ITEM_ID);

            FailedStepSearchCriteria criteria = new FailedStepSearchCriteria();
            criteria.setWorkItemId(workItemId);
            List<FailedStep> failedStepList = service.findBySearchCriteria(criteria);
            // a failed step for this error already exists so update the status
            if (failedStepList != null && failedStepList.size() == 1) {
                failedStep = failedStepList.get(0);
                failedStep.setStatus(FailedStep.FailedStepStatus.OPEN);
            }

        } catch (Exception e) {
            log.error("Error to load failed step object.", e);
        }

        return failedStep;
    }

    /**
     * Create the FailedStep object from the message
     *
     * @param msg failed step Message
     * @return FailedStep object
     */
    private FailedStep constructFailedStepFromMessage(AmqpMessage<String> msg) {
        FailedStep failedStep = null;

        try {
            failedStep = new FailedStep();
            Long processInstanceId = MessageUtil.getProperty(msg, MessageUtil.PROP_PROCESS_INSTANCE_ID);
            Long workItemId = MessageUtil.getProperty(msg, MessageUtil.PROP_WORK_ITEM_ID);
            String processId = MessageUtil.getProperty(msg, MessageUtil.PROP_PROCESS_ID);
            Long referenceId = MessageUtil.getProperty(msg, MessageUtil.PROP_REFERENCE_BID);
            String referenceKey = MessageUtil.getProperty(msg, MessageUtil.PROP_REFERENCE_KEY);
            String processName = MessageUtil.getProperty(msg, MessageUtil.PROP_PROCESS_NAME);
            String nodeName = MessageUtil.getProperty(msg, MessageUtil.PROP_KIE_NODE_NAME);
            Long nodeId = MessageUtil.getProperty(msg, MessageUtil.PROP_KIE_NODE_ID);

            failedStep.setNodeId(String.valueOf(nodeId));
            failedStep.setOriginalJMSMessage(msg.getPayload());
            failedStep.setHeaders(createHeadersText(msg));
            failedStep.setProcessInstanceId(processInstanceId);
            failedStep.setProcessId(processId);
            failedStep.setProcessName(processName);
            failedStep.setProcessStepName(nodeName);
            failedStep.setErrorCode(MessageUtil.getProperty(msg, MessageUtil.FAILED_STEP_ERROR_CODE));
            failedStep.setErrorMessage(MessageUtil.getProperty(msg, MessageUtil.FAILED_STEP_ERROR_MSG));
            failedStep.setDeepLink(MessageUtil.getProperty(msg, MessageUtil.FAILED_STEP_DEEP_LINK));
            failedStep.setReferenceBid("" + referenceId);
            failedStep.setReferenceKey(referenceKey);
            failedStep.setWorkItemId(workItemId);
            failedStep.setExecutionCount(0);
            failedStep.setResolutionOptionsString("RETRY");
            failedStep.setStatus(FailedStep.FailedStepStatus.OPEN);
            failedStep.setFailureType(FailedStep.FailureType.TECHNICAL_ERROR);
        } catch (Exception e) {
            log.error("Error creating the failed step object.", e);
        }

        return failedStep;
    }

    /**
     * Create Json text from all header parameters of a jms message
     *
     * @param message jms message
     * @return json string from the header parameters
     */
    private String createHeadersText(AmqpMessage<String> message) {
        String headerText = null;
        Map<String, Object> result = new HashMap<>();

        try {
            JsonObject properties = message.getApplicationProperties();
            result = properties.getMap();

            headerText = MessageUtil.serializeBody(result);
        } catch (Exception e) {
            log.error("Error serializing the header parameters of a message.", e);
        }

        return headerText;
    }

    private String constructResolutionCallbackUrlForCentralLog(String failedStepId) {
        StringBuilder urlBuilder = new StringBuilder();
        String callbackURlFromEnv = System.getenv(SUPPORT_TOOL_CALLBACK_URL_PROPERTY);
        if (callbackURlFromEnv != null) {
            urlBuilder.append(callbackURlFromEnv);
        } else {
            String appName = System.getenv("TKIT_SERVICE_NAME");
            appName = appName != null ? appName : "UNKNOWN";
            urlBuilder.append("http://");
            urlBuilder.append(appName);
            urlBuilder.append(contextPath);
            urlBuilder.append("/failedSteps/resolutionCallback");
        }
        urlBuilder.append("/onIssueResolvedCallback?failedStep=");
        urlBuilder.append(failedStepId);

        return urlBuilder.toString();
    }

    public enum FailedStepServiceErrorKeys {
        FAILED_TO_CREATE_FAILED_STEP
    }
}

==> tkit-quarkus-rhpam-client-0.0.15-sources.jar: org/tkit/rhpam/quarkus/messaging/FailedStepService.java <==
package org.tkit.rhpam.quarkus.messaging;

import ext.api.centrallog.api.ProcessLogEventEmitter;
import ext.api.centrallog.model.*;
import io.smallrye.reactive.messaging.amqp.AmqpMessage;
import io.vertx.core.json.JsonObject;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.eclipse.microprofile.reactive.messaging.Message;
import org.hibernate.service.spi.ServiceException;
import org.tkit.rhpam.quarkus.domain.daos.FailedStepDAO;
import org.tkit.rhpam.quarkus.domain.models.FailedStep;
import org.tkit.rhpam.quarkus.domain.models.FailedStepSearchCriteria;
import org.tkit.rhpam.quarkus.messaging.common.Constants;
import org.tkit.rhpam.quarkus.messaging.common.MessageUtil;
import org.tkit.rhpam.quarkus.messaging.common.RhpamException;
import org.tkit.rhpam.quarkus.messaging.model.ProcessStepExecution;
import org.tkit.rhpam.quarkus.messaging.model.ProcessStepExecutionResult;

import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * The type Failed step service.
 */
@ApplicationScoped
@Slf4j
public class FailedStepService {
    private static final String SUPPORT_TOOL_CALLBACK_URL_PROPERTY = "ORG_TKIT_RHPAM_SUPPORT_TOOL_CALLBACK_URL";
    @Inject
    ProcessLogEventEmitter centralLogEmitter;

    @ConfigProperty(name = "quarkus.resteasy.path")
    private String contextPath;

    @Inject
    FailedStepDAO service;

    public FailedStep createOrUpdateFailedStep(AmqpMessage<String> message) throws ServiceException {
        try {
            FailedStep failedStep = getExistingFailedStep(message);
            if (failedStep != null) {
                failedStep = service.update(failedStep);
            } else {
                failedStep = constructFailedStepFromMessage(message);
                failedStep = service.create(failedStep);
            }

            return failedStep;

        } catch (Exception e) {
            throw new IllegalStateException(FailedStepServiceErrorKeys.FAILED_TO_CREATE_FAILED_STEP.name(), e);
        }
    }

    public void emitFailedStepEvent(FailedStep failedStep) throws  RhpamException {
        OpenIncidentEvent event = new OpenIncidentEvent();
        event.setErrorCode(failedStep.getErrorCode());
        event.setErrorMessage(failedStep.getErrorMessage());
        event.setErrorType(ErrorType.fromValue(failedStep.getFailureType().name()));
        event.setNodeId(failedStep.getNodeId());
        event.setExternalIssueId(failedStep.getExternalIssueId());
        event.setPriority(PriorityLevel.HIGH);
        List<ResolutionOption> options = new ArrayList<>();
        String[] optionArr = failedStep.getResolutionOptionsString().split(",");
        for (String item : optionArr) {
            options.add(new ResolutionOption().key(item).label(item).callBackUrl(constructResolutionCallbackUrlForCentralLog(failedStep.getId())));
        }
        event.setResolutionOptions(options);
        event.setRetryCount(failedStep.getExecutionCount());
        event.setCorrelationId(failedStep.getId());
        event.setEventTime(failedStep.getCreationDate());
        event.setIncidentId(failedStep.getId());
        event.setProcessEventType(EventType.OPEN_INCIDENT_EVENT);
        event.setProcessId(failedStep.getProcessId());
        event.setProcessInstanceId(String.valueOf(failedStep.getProcessInstanceId()));
        event.setExecutionId(String.valueOf(failedStep.getWorkItemId()));
        event.getMetadata().put("deepLink", failedStep.getDeepLink());
        event.getMetadata().put("deploymentId", failedStep.getDeploymentId());
        event.getMetadata().put("referenceKey", failedStep.getReferenceKey());
        event.getMetadata().put("referenceBid", failedStep.getReferenceBid());
        event.getMetadata().put("processStepName", failedStep.getProcessStepName());
        centralLogEmitter.emitProcessEvent(event);
    }

    /**
     * Create Failed step for a process step that was technically executed ok, but resulted in business error
     *
     * @param workItem        process execution data
     * @param executionResult process execution result data
     * @return created failed step
     * @throws ServiceException
     */
    public FailedStep createOrUpdateBusinessFailedStep(ProcessStepExecution workItem, ProcessStepExecutionResult executionResult) throws ServiceException, IOException {
        FailedStep failedStep = null;
        FailedStepSearchCriteria criteria = new FailedStepSearchCriteria();
        criteria.setFailureType(FailedStep.FailureType.BUSINESS_ERROR);
        criteria.setProcessStepName(workItem.getName());
        criteria.setProcessInstanceId(workItem.getProcessInstanceId());
        List<FailedStep> failedStepList = service.findBySearchCriteria(criteria);
        // a failed step for this error already exists so update the status
        if (failedStepList != null && failedStepList.size() == 1) {
            failedStep = failedStepList.get(0);
            failedStep.setExecutionCount(failedStep.getExecutionCount() + 1);
            failedStep.setStatus(FailedStep.FailedStepStatus.OPEN);
            failedStep.setWorkItemId(workItem.getWorkItemId());
            return service.update(failedStep);
        } else {
            failedStep = new FailedStep();
            failedStep.setFailureType(FailedStep.FailureType.BUSINESS_ERROR);

            //failedStep.setResolutionOptionsString(resolutionOptions);
            //failedStep.setResolutionSignalReceiver(resolutionSignalId);
            failedStep.setExecutionCount(1);
            failedStep.setProcessInstanceId(workItem.getProcessInstanceId());

            if (executionResult.getParameters().get(Constants.FAILED_STEP_ERROR_CODE) != null) {
                failedStep.setErrorCode((String) executionResult.getParameters().get(Constants.FAILED_STEP_ERROR_CODE));
            }
            if (executionResult.getParameters().get(Constants.FAILED_STEP_ERROR_MSG) != null) {
                failedStep.setErrorMessage((String) executionResult.getParameters().get(Constants.FAILED_STEP_ERROR_MSG));
            }
            if (executionResult.getParameters().get(Constants.FAILED_STEP_DEEP_LINK) != null) {
                failedStep.setDeepLink((String) executionResult.getParameters().get(Constants.FAILED_STEP_DEEP_LINK));
            }
            failedStep.setNodeId(workItem.getNodeId());
            failedStep.setNodeName(workItem.getName());
            failedStep.setProcessId(workItem.getProcessId());
            failedStep.setProcessInstanceId(workItem.getProcessInstanceId());
            failedStep.setParentProcessInstanceId(workItem.getParentProcessInstanceId() != null ? String.valueOf(workItem.getParentProcessInstanceId()) : null);
            failedStep.setProcessStepName(workItem.getName());
            failedStep.setProcessName(workItem.getProcessName());
            failedStep.setStatus(FailedStep.FailedStepStatus.OPEN);
            failedStep.setWorkItemId(workItem.getWorkItemId());
            failedStep.setReferenceBid(workItem.getReferenceId());
            failedStep.setReferenceKey(workItem.getReferenceKey());
            failedStep.setDeploymentId(workItem.getDeploymentId());
            //TODO hack, we misuses the field, to store the output params, because we do not have other option
            failedStep.setOriginalJMSMessage(MessageUtil.serializeBody(executionResult.getParameters()));
            return service.create(failedStep);
        }
    }

    public FailedStep updateFailedStep(ProcessStepExecution workItem, String referencedStepName, String resolutionOptions, String resolutionSignalId) throws ServiceException {
        FailedStep failedStep = null;
        FailedStepSearchCriteria criteria = new FailedStepSearchCriteria();
        criteria.setFailureType(FailedStep.FailureType.BUSINESS_ERROR);
        criteria.setProcessStepName(referencedStepName);
        criteria.setProcessInstanceId(workItem.getProcessInstanceId());
        List<FailedStep> failedStepList = service.findBySearchCriteria(criteria);
        // a failed step for this error already exists so update the status
        if (failedStepList != null && failedStepList.size() == 1) {
            failedStep = failedStepList.get(0);
        } else {
            throw new IllegalArgumentException("Business error handler invoked but no matching failed step found! Step: " +  referencedStepName);
        }
        failedStep.setResolutionOptionsString(resolutionOptions);
        failedStep.setResolutionSignalReceiver(resolutionSignalId);
        failedStep = service.update(failedStep);

        return failedStep;
    }

    /**
     * Try to find the FailedStep object
     *
     * @param msg failed step Message
     * @return found FailedStep based on workItemId or null with updated status
     */
    private FailedStep getExistingFailedStep(Message msg) {
        FailedStep failedStep = null;

        try {

            Long workItemId = MessageUtil.getProperty(msg, MessageUtil.PROP_WORK_ITEM_ID);

            FailedStepSearchCriteria criteria = new FailedStepSearchCriteria();
            criteria.setWorkItemId(workItemId);
            List<FailedStep> failedStepList = service.findBySearchCriteria(criteria);
            // a failed step for this error already exists so update the status
            if (failedStepList != null && failedStepList.size() == 1) {
                failedStep = failedStepList.get(0);
                failedStep.setStatus(FailedStep.FailedStepStatus.OPEN);
            }

        } catch (Exception e) {
            log.error("Error to load failed step object.", e);
        }

        return failedStep;
    }

    /**
     * Create the FailedStep object from the message
     *
     * @param msg failed step Message
     * @return FailedStep object
     */
    private FailedStep constructFailedStepFromMessage(AmqpMessage<String> msg) {
        FailedStep failedStep = null;

        try {
            failedStep = new FailedStep();
            Long processInstanceId = MessageUtil.getProperty(msg, MessageUtil.PROP_PROCESS_INSTANCE_ID);
            Long workItemId = MessageUtil.getProperty(msg, MessageUtil.PROP_WORK_ITEM_ID);
            String processId = MessageUtil.getProperty(msg, MessageUtil.PROP_PROCESS_ID);
            Long referenceId = MessageUtil.getProperty(msg, MessageUtil.PROP_REFERENCE_BID);
            String referenceKey = MessageUtil.getProperty(msg, MessageUtil.PROP_REFERENCE_KEY);
            String processName = MessageUtil.getProperty(msg, MessageUtil.PROP_PROCESS_NAME);
            String nodeName = MessageUtil.getProperty(msg, MessageUtil.PROP_KIE_NODE_NAME);
            Long nodeId = MessageUtil.getProperty(msg, MessageUtil.PROP_KIE_NODE_ID);

            failedStep.setNodeId(String.valueOf(nodeId));
            failedStep.setOriginalJMSMessage(msg.getPayload());
            failedStep.setHeaders(createHeadersText(msg));
            failedStep.setProcessInstanceId(processInstanceId);
            failedStep.setProcessId(processId);
            failedStep.setProcessName(processName);
            failedStep.setProcessStepName(nodeName);
            failedStep.setErrorCode(MessageUtil.getProperty(msg, MessageUtil.FAILED_STEP_ERROR_CODE));
            failedStep.setErrorMessage(MessageUtil.getProperty(msg, MessageUtil.FAILED_STEP_ERROR_MSG));
            failedStep.setDeepLink(MessageUtil.getProperty(msg, MessageUtil.FAILED_STEP_DEEP_LINK));
            failedStep.setReferenceBid("" + referenceId);
            failedStep.setReferenceKey(referenceKey);
            failedStep.setWorkItemId(workItemId);
            failedStep.setExecutionCount(0);
            failedStep.setResolutionOptionsString("RETRY");
            failedStep.setStatus(FailedStep.FailedStepStatus.OPEN);
            failedStep.setFailureType(FailedStep.FailureType.TECHNICAL_ERROR);
        } catch (Exception e) {
            log.error("Error creating the failed step object.", e);
        }

        return failedStep;
    }

    /**
     * Create Json text from all header parameters of a jms message
     *
     * @param message jms message
     * @return json string from the header parameters
     */
    private String createHeadersText(AmqpMessage<String> message) {
        String headerText = null;
        Map<String, Object> result = new HashMap<>();

        try {
            JsonObject properties = message.getApplicationProperties();
            result = properties.getMap();

            headerText = MessageUtil.serializeBody(result);
        } catch (Exception e) {
            log.error("Error serializing the header parameters of a message.", e);
        }

        return headerText;
    }

    private String constructResolutionCallbackUrlForCentralLog(String failedStepId) {
        StringBuilder urlBuilder = new StringBuilder();
        String callbackURlFromEnv = System.getenv(SUPPORT_TOOL_CALLBACK_URL_PROPERTY);
        if (callbackURlFromEnv != null) {
            urlBuilder.append(callbackURlFromEnv);
        } else {
            String appName = System.getenv("TKIT_SERVICE_NAME");
            appName = appName != null ? appName : "UNKNOWN";
            urlBuilder.append("http://");
            urlBuilder.append(appName);
            urlBuilder.append(contextPath);
            urlBuilder.append("/failedSteps/resolutionCallback");
        }
        urlBuilder.append("/onIssueResolvedCallback?failedStep=");
        urlBuilder.append(failedStepId);

        return urlBuilder.toString();
    }

    public enum FailedStepServiceErrorKeys {
        FAILED_TO_CREATE_FAILED_STEP
    }
}
